#! /usr/bin/env python
# coding=utf-8

'''
Example:
# Infer
CUDA_VISIBLE_DEVICES=0  python letrain.py --model_name=yolov3 --inference=true --checkpoint_dir=/home/wangtao34/train/yolov3_2/ --input_dir=/home/wangtao34/dataset/image/voc/VOC2007TEST/ --output_dir=/home/wangtao34/train/yolov3test --mAP=True
# optional: --mAP=true
'''
import math
import operator
import glob
import json
import cv2
import os
import sys
import numpy as np
import tensorflow as tf
import matplotlib.pyplot as plt
import shutil
from scipy.misc import imread, imsave, imshow, imresize

parentpath = os.path.dirname(os.path.abspath(__file__))
path_list = [parentpath[:-16] + 'applications',
             parentpath[:-16] + 'applications/' + 'yolov3',
             parentpath[:-6]]
sys.path += path_list
print(sys.path)

from core.yolov3 import YOLOV3
import core.utils as utils

from  mAP.get_gt_txt import get_gt_txt
from  mAP.main import mAP

FLAGS = tf.app.flags.FLAGS

if os.path.exists(FLAGS.output_dir + "/input"): shutil.rmtree(FLAGS.output_dir + "/input")

if os.path.exists(FLAGS.output_dir + "/input"):
    os.makedirs(FLAGS.output_dir + "/input")
if not os.path.exists(FLAGS.output_dir + "/input/detection-results"):
    os.makedirs(FLAGS.output_dir + "/input/detection-results")
if not os.path.exists(FLAGS.output_dir + "/input/images-optional"):
    os.makedirs(FLAGS.output_dir + "/input/images-optional")
if not os.path.exists(FLAGS.output_dir + "/input/ground-truth"):
    os.makedirs(FLAGS.output_dir + "/input/ground-truth")

# tf.app.flags.DEFINE_boolean("use_07_metric", False, "Whether to use VOC07's 11 point AP computation")

return_elements = ["input/input_data:0", "pred_sbbox/concat_2:0", "pred_mbbox/concat_2:0", "pred_lbbox/concat_2:0"]
classes = utils.read_class_names(FLAGS.checkpoint_dir)
num_classes = len(classes)
input_size = 544


def get_checkpoint():
    if tf.gfile.IsDirectory(FLAGS.checkpoint_dir):
        checkpoint_dir = tf.train.latest_checkpoint(FLAGS.checkpoint_dir)
    else:
        checkpoint_dir = FLAGS.checkpoint_dir

    return checkpoint_dir


def main():
    with tf.name_scope('input'):
        input_data = tf.placeholder(dtype=tf.float32, name='input_data')

    with tf.Session(config=tf.ConfigProto(allow_soft_placement=True)) as sess:
        if FLAGS.model_compress:
            print('using model compressing!')
            saver = tf.train.import_meta_graph(get_checkpoint() + '.meta')
            saver.restore(sess, get_checkpoint())
            _pred_sbbox = tf.get_default_graph().get_tensor_by_name('define_loss/pred_sbbox/concat_2:0')
            _pred_mbbox = tf.get_default_graph().get_tensor_by_name('define_loss/pred_mbbox/concat_2:0')
            _pred_lbbox = tf.get_default_graph().get_tensor_by_name('define_loss/pred_lbbox/concat_2:0')
            input_data = tf.get_default_graph().get_tensor_by_name("IteratorGetNext:0")
        else:
            print('starting inference!')
            model = YOLOV3(input_data, trainable=False)
            print(model.conv_sbbox, model.conv_mbbox, model.conv_lbbox)
            _pred_sbbox, _pred_mbbox, _pred_lbbox = model.pred_sbbox, model.pred_mbbox, model.pred_lbbox
            saver = tf.train.Saver()
            init = tf.global_variables_initializer()
            sess.run(init)
            saver.restore(sess, get_checkpoint())

        input_dir = FLAGS.input_dir
        if not input_dir.endswith('/'):
            input_dir = input_dir + '/'
        for dirpath, dirnames, filenames in os.walk(input_dir):
            if dirnames != [] and FLAGS.mAP == False:
                print 'The test folder should contain only images, not folders'
                os._exit(0)

            if FLAGS.mAP:
                detections = {}
                file_name = []
                image_dir = os.path.join(input_dir, 'JPEGImages')
                ann_path = os.path.join(input_dir, 'Annotations')
                filenames = os.listdir(image_dir)
                save_path = FLAGS.output_dir + "/input/ground-truth"
                for i, filename in enumerate(filenames):
                    p = input_dir + 'JPEGImages/' + filename
                    original_image = cv2.imread(p)
                    # original_image = cv2.cvtColor(original_image, cv2.COLOR_BGR2RGB)
                    original_image_size = original_image.shape[:2]
                    image_data = utils.image_preporcess(np.copy(original_image), [input_size, input_size])
                    image_data = image_data[np.newaxis, ...]
                    # print('Demo for {}'.format(filename))
                    if i % 100 == 0:
                        print('Demo for {:d}/{:d}'.format(
                            i + 1, len(filenames)))
                    pred_sbbox, pred_mbbox, pred_lbbox = sess.run(
                        [_pred_sbbox, _pred_mbbox, _pred_lbbox],
                        feed_dict={input_data: image_data})

                    pred_bbox = np.concatenate([np.reshape(pred_sbbox, (-1, 5 + num_classes)),
                                                np.reshape(pred_mbbox, (-1, 5 + num_classes)),
                                                np.reshape(pred_lbbox, (-1, 5 + num_classes))], axis=0)
                    bboxes = utils.postprocess_boxes(pred_bbox, original_image_size, input_size, 0.3)
                    bboxes = utils.nms(bboxes, 0.45, method='nms')
                    filename = filename.split('.')[0]
                    f = open(FLAGS.output_dir + "/input/detection-results/" + filename + ".txt", "a")
                    for i, bbox in enumerate(bboxes):
                        predicted_class_ind = int(bbox[5])
                        predicted_class = classes[predicted_class_ind]
                        score = bbox[4]
                        left, top, right, bottom = np.array(bbox[:4], dtype=np.int32)
                        f.write("%s %s %s %s %s %s\n" % (
                        predicted_class, score, str(int(left)), str(int(top)), str(int(right)), str(int(bottom))))
                    f.close()
                    ann_file = os.path.join(ann_path, filename+'.xml')
                    get_gt_txt(ann_file, save_path, filename, use_difficult_bbox=False)
                # execfile(os.path.join(sys.path[-1], 'mAP', 'main.py'))
                mAP(FLAGS.output_dir)
                break
            else:
                for filename in filenames:
                    p = input_dir + filename
                    original_image = cv2.imread(p)
                    original_image_size = original_image.shape[:2]
                    image_data = utils.image_preporcess(np.copy(original_image), [input_size, input_size])
                    image_data = image_data[np.newaxis, ...]
                    print('Demo for {}'.format(filename))
                    pred_sbbox, pred_mbbox, pred_lbbox = sess.run(
                        [_pred_sbbox, _pred_mbbox, _pred_lbbox],
                        feed_dict={input_data: image_data})

                    pred_bbox = np.concatenate([np.reshape(pred_sbbox, (-1, 5 + num_classes)),
                                                np.reshape(pred_mbbox, (-1, 5 + num_classes)),
                                                np.reshape(pred_lbbox, (-1, 5 + num_classes))], axis=0)

                    bboxes = utils.postprocess_boxes(pred_bbox, original_image_size, input_size, 0.3)
                    bboxes = utils.nms(bboxes, 0.45, method='nms')
                    original_image = cv2.cvtColor(original_image, cv2.COLOR_BGR2RGB)
                    image = utils.draw_bbox(original_image, bboxes)
                    output_dir = FLAGS.output_dir
                    if not tf.gfile.Exists(output_dir):
                        tf.gfile.MakeDirs(output_dir)
                    # filename_1, filename_2 = filename.split(".")
                    # ff = filename_1 + "_predict." + filename_2
                    ff = 'predict_' + filename
                    if not output_dir.endswith('/'):
                        output_dir = output_dir + '/'
                    output_name = output_dir + ff
                    imsave(output_name, image)
                    print("Saved to {}".format(output_name))

                # image = Image.fromarray(image)
                # image.show()


if __name__ == '__main__':
    tf.logging.set_verbosity(tf.logging.INFO)
    tf.app.run()
